import os
import sys
import json
from pathlib import Path

from .download import Download
DRIVER = Path(__file__).resolve().absolute().parent / "driver"

download = Download()


def get_installed_chrome_path():
    config_file = os.path.join(DRIVER, "config.json")
    if os.path.isfile(config_file):
        chromedriver = json.load(open(config_file, "r"))["chromedriver"]
        if chromedriver:
            return chromedriver
    chrome_version = download.check_installed_chrome_version()
    if chrome_version:
        chrome_driver_name = download.get_chrome_driver_download_link(chrome_version)
        download.download_chrome_driver(chrome_driver_name)
        filename = download.extract_chrome_driver_zip(chrome_driver_name)
        chromedriver = os.path.join(DRIVER, filename)
        return chromedriver
    else:
        return None


import shutil
import requests
import os
import platform
import subprocess
import re
import zipfile
from pathlib import Path
from bs4 import BeautifulSoup
from termcolor import colored
from colorama import init


from .utils import Util

OSNAME = platform.system()
init()
DRIVER = Path(__file__).resolve().absolute().parent / "driver"


class Download(Util):
    """
    It will get the installed Chrome driver and based on the operating system
    it will download the compatible chromedriver.
    """

    if OSNAME == "Windows":
        system = "win"
    elif OSNAME == "Linux":
        system = "linux"
    else:
        system = "mac"

    def update_binary(self, path):
        word = "cdc_".encode()
        new = "tch_".encode()
        while True:
            string = b""
            Flag = 0
            with open(path, 'r+b') as file:
                pos = 0
                data = string = file.read(1)
                while data:
                    data = file.read(1)
                    if data == b" ":
                        if word in string:
                            new_tring = string.decode().replace(word.decode(), new.decode())
                            file.seek(pos)
                            file.write(new_tring.encode())
                            Flag = 1
                            break
                        else:
                            pos = file.tell()
                            data = string = file.read(1)
                    else:
                        string += data
                        continue
            if not Flag:
                break

    def remove_signature_in_javascript(self, path):
        if OSNAME == "Windows":
            os.chmod(path, 755)
        else:
            subprocess.Popen(f"sudo chmod 777 {path}", stdout=subprocess.PIPE, shell=True)
        with open(path, "r", errors="ignore") as chrome:
            content = chrome.read()
        content = content.replace("cdc_", "tch_")   
        self.update_binary(path)


    def check_installed_chrome_version(self):
        try:
            if OSNAME == "Windows":
                cmd_version_output = (
                    subprocess.Popen(
                        'reg query "HKEY_CURRENT_USER\Software\Google\Chrome\BLBeacon" /v version',
                        shell=True,
                        stdout=subprocess.PIPE,
                    )
                    .stdout.read()
                    .decode("utf-8")
                )
            elif OSNAME == "Linux":
                cmd_version_output = (
                    subprocess.Popen(
                        "google-chrome --version", shell=True, stdout=subprocess.PIPE
                    )
                    .stdout.read()
                    .decode("utf-8")
                )
            elif OSNAME == "Darwin":
                cmd_version_output = (
                    subprocess.Popen(
                        "/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome --version",
                        shell=True,
                        stdout=subprocess.PIPE,
                    )
                    .stdout.read()
                    .decode("utf-8")
                )
        except Exception as error:
            print(
                colored("X [Error]", "red")
                + " We couldn't find the version of the installed chrome browser."
            )
            print(f"--> {error}")
            return None
        else:
            installed_chrome_version = re.findall(
                    r"([\d]+\.[\d]+\.[\d]+\.[\d]+)", cmd_version_output
                )[0].split(".")[0]
            print(
                colored("[+]", "green")
                + " You have Chrome version "
                + colored(installed_chrome_version, "blue")
                + " installed."
            )
            self.update_chrome_driver_config(version=installed_chrome_version)
            return installed_chrome_version

    def get_chrome_driver_download_link(self, version):
        base_url = "https://chromedriver.storage.googleapis.com"
        headers = {
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36"
        }
        response = requests.get(base_url, headers=headers)
        content = response.content.decode("utf-8")
        soup = BeautifulSoup(content, features="xml")
        keys = [key.text for key in soup.find_all("Key")]
        try:
            return [key for key in keys if self.system in key and version in key][0]
        except IndexError:
            return None

    def download_chrome_driver(self, chrome_driver_file):
        base_url = "https://chromedriver.storage.googleapis.com"
        headers = {
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36"
        }
        download_link = base_url + "/" + chrome_driver_file
        chrome_file_name = chrome_driver_file.split("/")[1]
        with requests.get(download_link, stream=True, headers=headers) as r:
            print("downloading " + colored(f"{chrome_file_name}", "blue") + " ...")
            with open(os.path.join(DRIVER, chrome_file_name), "wb") as f:
                shutil.copyfileobj(r.raw, f)
        
        return chrome_driver_file

    def extract_chrome_driver_zip(self, chrome_driver_file):
        global filename
        chrome_file_name = chrome_driver_file.split("/")[1]
        path = os.path.join(DRIVER, chrome_file_name)
        with zipfile.ZipFile(path, "r") as zip_ref:
            zip_ref.extractall(DRIVER)
            filename = zip_ref.namelist()[0]
        os.remove(path)
        print(colored("[+]", "green") + " Chrome driver has been installed.")
        chromedriver_path = os.path.join(DRIVER, filename)
        self.update_chrome_driver_config(chromedriver=chromedriver_path)
        
        self.remove_signature_in_javascript(chromedriver_path)
        return filename


import sys
sys.path.append("...")
from lucd.driver import Driver

def test_create_driver():
    driver = Driver()
    chrome = driver.create_driver(headless=True)
    assert chrome.title == "Blog posts"
    chrome.quit()



from selenium import webdriver
from selenium.webdriver.chrome.service import Service
import random
import os
import platform
from fake_useragent import UserAgent
import sys
from termcolor import colored
from colorama import init
init()
# import Keys
from selenium.webdriver.common.keys import Keys


from .utils import Util


class Driver(Util):

    def generate_user_agent(self):
        try:
            ua = UserAgent()
            user_agent = ua['google chrome']
        except IndexError:
            user_agent = random.choice([          
                "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.45 Safari/537.36",
                "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.45 Safari/537.36",
                "Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.45 Safari/537.36",
            ])
        return user_agent

    def driver_navigator(self, driver):
        if driver.execute_script("return navigator.webdriver"):
            driver.execute_cdp_cmd(
                    "Page.addScriptToEvaluateOnNewDocument",
                    {
                        "source": """

                                Object.defineProperty(window, 'navigator', {
                                    value: new Proxy(navigator, {
                                    has: (target, key) => (key === 'webdriver' ? false : key in target),
                                    get: (target, key) =>
                                        key === 'webdriver'
                                        ? undefined
                                        : typeof target[key] === 'function'
                                        ? target[key].bind(target)
                                        : target[key]
                                    })
                                });
                                    
                                                            
                        """
                    },
                )

    def open_new_cmd_and_run_command(self, profile_path="", port=9222, default_profile=False):
        """
        This function opens a new cmd window and runs the command
        a command that runs the Chrome driver in the debugging mode
        """
        system = platform.system()
        pwd = os.getcwd()
        if not profile_path and not default_profile:
            profile_path = pwd + f"/profile{random.randint(1,100)}"
        if profile_path and system == 'Windows':
            profile_path = profile_path.replace('/', '\\') if "/" in profile_path else profile_path
        if default_profile:
            command = f"chrome.exe -remote-debugging-port={port}"
        else:
            command = f'chrome.exe --remote-debugging-port={port} --user-data-dir="{profile_path}"'
        if system == 'Windows':
            os.system('start cmd /k ' + command)
        else:
            google = self.whereis_google_chrome()
            if not google:
                raise Exception("whereis: Google Chrome is not installed")
            if default_profile:
                command = f'{google} --remote-debugging-port={port}'
            else:
                command = f'{google} --remote-debugging-port={port} --user-data-dir="{profile_path}"'
            os.system(f'gnome-terminal -- bash -c "{command}"')

    def create_driver(self, headless=False, profile_path="", mute=False, debugging=False, default_profile=False, debug_port=9222, control_existing_instance=False):
        from .check import get_installed_chrome_path
        path = get_installed_chrome_path()
        if path is not None:
            options = webdriver.ChromeOptions()
            options.headless = headless
            options.add_argument("--start-maximized")
            options.add_argument("--lang=en-US")
            if profile_path:
                options.add_argument(r"--user-data-dir=%s" % profile_path)
            if debugging:
                if not control_existing_instance:
                    self.open_new_cmd_and_run_command(profile_path=profile_path, port=debug_port, default_profile=default_profile)
                options.add_experimental_option("debuggerAddress", f"127.0.0.1:{debug_port}")
            else:
                options.add_experimental_option("excludeSwitches", ["enable-automation", "enable-logging"])
                options.add_argument("--log-level=3")
                options.add_experimental_option('useAutomationExtension', False)
                options.add_argument(f"user-agent={self.generate_user_agent()}")
                if mute:
                    options.add_argument("--mute-audio")
                    options.add_argument('--no-sandbox')
                options.add_argument("start-maximized")
                options.add_argument("--disable-blink-features=AutomationControlled")
            try:
                driver = webdriver.Chrome(service=Service(path), options=options)
            except TypeError:
                driver = webdriver.Chrome(executable_path=path, options=options)
            except Exception as error:
                sys.exit(colored("[-] ", "red")+f" {error}")
            else:
                self.driver_navigator(driver)
                driver.get("https://selmi.tech")
                return driver
        else:
            sys.exit(colored(f"Make sure you have chrome installed on your machine!", "red"))



from setuptools import setup, find_packages

VERSION = '0.1.7' 
DESCRIPTION = 'Create Local Undetectable Chrome Driver'
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setup(
        name="lucd", 
        version=VERSION,
        author="Selmi Abderrahim",
        author_email="contact@selmi.tech",
        description=DESCRIPTION,
        long_description=long_description,
        long_description_content_type="text/markdown",
        url='https://github.com/SelmiAbderrahim/CreateUndetectableChromeDriver',
        install_requires=[
            "beautifulsoup4==4.11.1",
            "colorama==0.4.4",
            "requests==2.27.1",
            "termcolor==1.1.0",
            "lxml==4.9.1",
            "selenium==4.1.5",
            "fake-useragent==0.1.11",
        ],
        packages=['lucd', 'lucd.screenshots', 'lucd.tests'],
        keywords=['python', 'undetectable', "chrome", "selenium", "webdriver"],
        classifiers= [
            "Development Status :: 4 - Beta",
            "Programming Language :: Python :: 3",
            "Operating System :: Microsoft :: Windows",
        ],
        license='MIT',
)



import json
import os
import platform
from pathlib import Path
import subprocess

DRIVER = Path(__file__).resolve().absolute().parent / "driver"


class Util:
    def create_chrome_driver_config(self):
        if not os.path.isdir(DRIVER):
            os.mkdir(DRIVER)
        config_file = os.path.join(DRIVER, "config.json")
        if not os.path.isfile(config_file):
            with open(config_file, "w") as config:
                data = {
                    "system":platform.system(),
                    "version":"",
                    "chromedriver":""
                }
                config.write(json.dumps(data))
        return config_file

    def update_chrome_driver_config(self, version="", chromedriver=""):
        config_file = self.create_chrome_driver_config()
        config_data = json.load(open(config_file, "r"))
        if version:
            config_data["version"]=version
        if chromedriver:
            config_data["chromedriver"] = chromedriver
        with open(config_file, "w") as config:
            config.write(json.dumps(config_data))

    def whereis_google_chrome(self):
        """
        Get the path of the chrome driver
        """
        try:
            chrome_path = subprocess.check_output(['whereis', 'google-chrome']).decode('utf-8').strip().split()[1]
        except:
            return False
        return chrome_path